package javax.swing.plaf.synth;

import java.beans.PropertyChangeEvent;
import java.awt.Dimension;
import javax.swing.Icon;
import java.awt.Graphics;
import java.awt.FontMetrics;
import javax.swing.plaf.basic.BasicHTML;
import javax.swing.text.View;
import java.awt.Rectangle;
import javax.swing.ButtonModel;
import javax.swing.JButton;
import java.awt.Insets;
import javax.swing.plaf.UIResource;
import javax.swing.LookAndFeel;
import javax.swing.AbstractButton;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicButtonUI;

public class SynthButtonUI extends BasicButtonUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    
    public SynthButtonUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthButtonUI();
    }
    
    @Override
    protected void installDefaults(AbstractButton c) {
        this.updateStyle(c);
        LookAndFeel.installProperty(c, "rolloverEnabled", Boolean.TRUE);
    }
    
    @Override
    protected void installListeners(AbstractButton b) {
        super.installListeners(b);
        b.addPropertyChangeListener(this);
    }
    
    void updateStyle(AbstractButton abstractButton) {
        SynthContext context = this.getContext(abstractButton, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context, this);
        if (this.style != style) {
            if (abstractButton.getMargin() == null || abstractButton.getMargin() instanceof UIResource) {
                Insets empty_UIRESOURCE_INSETS = (Insets)this.style.get(context, this.getPropertyPrefix() + "margin");
                if (empty_UIRESOURCE_INSETS == null) {
                    empty_UIRESOURCE_INSETS = SynthLookAndFeel.EMPTY_UIRESOURCE_INSETS;
                }
                abstractButton.setMargin(empty_UIRESOURCE_INSETS);
            }
            Object value = this.style.get(context, this.getPropertyPrefix() + "iconTextGap");
            if (value != null) {
                LookAndFeel.installProperty(abstractButton, "iconTextGap", value);
            }
            Object value2 = this.style.get(context, this.getPropertyPrefix() + "contentAreaFilled");
            LookAndFeel.installProperty(abstractButton, "contentAreaFilled", (value2 != null) ? value2 : Boolean.TRUE);
            if (style != null) {
                this.uninstallKeyboardActions(abstractButton);
                this.installKeyboardActions(abstractButton);
            }
        }
        context.dispose();
    }
    
    @Override
    protected void uninstallListeners(AbstractButton b) {
        super.uninstallListeners(b);
        b.removePropertyChangeListener(this);
    }
    
    @Override
    protected void uninstallDefaults(AbstractButton abstractButton) {
        SynthContext context = this.getContext(abstractButton, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, this.getComponentState(component));
    }
    
    SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private int getComponentState(JComponent component) {
        int n = 1;
        if (!component.isEnabled()) {
            n = 8;
        }
        if (SynthLookAndFeel.getSelectedUI() == this) {
            return SynthLookAndFeel.getSelectedUIState() | 0x1;
        }
        AbstractButton abstractButton = (AbstractButton)component;
        ButtonModel model = abstractButton.getModel();
        if (model.isPressed()) {
            if (model.isArmed()) {
                n = 4;
            }
            else {
                n = 2;
            }
        }
        if (model.isRollover()) {
            n |= 0x2;
        }
        if (model.isSelected()) {
            n |= 0x200;
        }
        if (component.isFocusOwner() && abstractButton.isFocusPainted()) {
            n |= 0x100;
        }
        if (component instanceof JButton && ((JButton)component).isDefaultButton()) {
            n |= 0x400;
        }
        return n;
    }
    
    @Override
    public int getBaseline(JComponent component, int n, int n2) {
        if (component == null) {
            throw new NullPointerException("Component must be non-null");
        }
        if (n < 0 || n2 < 0) {
            throw new IllegalArgumentException("Width and height must be >= 0");
        }
        AbstractButton abstractButton = (AbstractButton)component;
        String text = abstractButton.getText();
        if (text == null || "".equals(text)) {
            return -1;
        }
        Insets insets = abstractButton.getInsets();
        Rectangle rectangle = new Rectangle();
        Rectangle rectangle2 = new Rectangle();
        Rectangle rectangle3 = new Rectangle();
        rectangle.x = insets.left;
        rectangle.y = insets.top;
        rectangle.width = n - (insets.right + rectangle.x);
        rectangle.height = n2 - (insets.bottom + rectangle.y);
        SynthContext context = this.getContext(abstractButton);
        FontMetrics fontMetrics = context.getComponent().getFontMetrics(context.getStyle().getFont(context));
        context.getStyle().getGraphicsUtils(context).layoutText(context, fontMetrics, abstractButton.getText(), abstractButton.getIcon(), abstractButton.getHorizontalAlignment(), abstractButton.getVerticalAlignment(), abstractButton.getHorizontalTextPosition(), abstractButton.getVerticalTextPosition(), rectangle, rectangle3, rectangle2, abstractButton.getIconTextGap());
        View view = (View)abstractButton.getClientProperty("html");
        int htmlBaseline;
        if (view != null) {
            htmlBaseline = BasicHTML.getHTMLBaseline(view, rectangle2.width, rectangle2.height);
            if (htmlBaseline >= 0) {
                htmlBaseline += rectangle2.y;
            }
        }
        else {
            htmlBaseline = rectangle2.y + fontMetrics.getAscent();
        }
        context.dispose();
        return htmlBaseline;
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        this.paintBackground(context, graphics, component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
        AbstractButton abstractButton = (AbstractButton)synthContext.getComponent();
        graphics.setColor(synthContext.getStyle().getColor(synthContext, ColorType.TEXT_FOREGROUND));
        graphics.setFont(this.style.getFont(synthContext));
        synthContext.getStyle().getGraphicsUtils(synthContext).paintText(synthContext, graphics, abstractButton.getText(), this.getIcon(abstractButton), abstractButton.getHorizontalAlignment(), abstractButton.getVerticalAlignment(), abstractButton.getHorizontalTextPosition(), abstractButton.getVerticalTextPosition(), abstractButton.getIconTextGap(), abstractButton.getDisplayedMnemonicIndex(), this.getTextShiftOffset(synthContext));
    }
    
    void paintBackground(SynthContext synthContext, Graphics graphics, JComponent component) {
        if (((AbstractButton)component).isContentAreaFilled()) {
            synthContext.getPainter().paintButtonBackground(synthContext, graphics, 0, 0, component.getWidth(), component.getHeight());
        }
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintButtonBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    protected Icon getDefaultIcon(AbstractButton abstractButton) {
        SynthContext context = this.getContext(abstractButton);
        Icon icon = context.getStyle().getIcon(context, this.getPropertyPrefix() + "icon");
        context.dispose();
        return icon;
    }
    
    protected Icon getIcon(AbstractButton abstractButton) {
        Icon icon = abstractButton.getIcon();
        ButtonModel model = abstractButton.getModel();
        Icon icon2;
        if (!model.isEnabled()) {
            icon2 = this.getSynthDisabledIcon(abstractButton, icon);
        }
        else if (model.isPressed() && model.isArmed()) {
            icon2 = this.getPressedIcon(abstractButton, this.getSelectedIcon(abstractButton, icon));
        }
        else if (abstractButton.isRolloverEnabled() && model.isRollover()) {
            icon2 = this.getRolloverIcon(abstractButton, this.getSelectedIcon(abstractButton, icon));
        }
        else if (model.isSelected()) {
            icon2 = this.getSelectedIcon(abstractButton, icon);
        }
        else {
            icon2 = this.getEnabledIcon(abstractButton, icon);
        }
        if (icon2 == null) {
            return this.getDefaultIcon(abstractButton);
        }
        return icon2;
    }
    
    private Icon getIcon(AbstractButton abstractButton, Icon synthIcon, Icon icon, int n) {
        if (synthIcon == null) {
            if ((icon instanceof UIResource)) {
                synthIcon = this.getSynthIcon(abstractButton, n);
                if (synthIcon == null) {
                    synthIcon = icon;
                }
            }
            else {
                synthIcon = icon;
            }
        }
        return synthIcon;
    }
    
    private Icon getSynthIcon(AbstractButton abstractButton, int n) {
        return this.style.getIcon(this.getContext(abstractButton, n), this.getPropertyPrefix() + "icon");
    }
    
    private Icon getEnabledIcon(AbstractButton abstractButton, Icon synthIcon) {
        if (synthIcon == null) {
            synthIcon = this.getSynthIcon(abstractButton, 1);
            return synthIcon;
        }
        return synthIcon;
    }
    
    private Icon getSelectedIcon(AbstractButton abstractButton, Icon icon) {
        return this.getIcon(abstractButton, abstractButton.getSelectedIcon(), icon, 512);
    }
    
    private Icon getRolloverIcon(AbstractButton abstractButton, Icon icon) {
        if (abstractButton.getModel().isSelected()) {
            return this.getIcon(abstractButton, abstractButton.getRolloverSelectedIcon(), icon, 514);
        }
        return this.getIcon(abstractButton, abstractButton.getRolloverIcon(), icon, 2);
    }
    
    private Icon getPressedIcon(AbstractButton abstractButton, Icon icon) {
        return this.getIcon(abstractButton, abstractButton.getPressedIcon(), icon, 4);
    }
    
    private Icon getSynthDisabledIcon(AbstractButton abstractButton, Icon icon) {
        if (abstractButton.getModel().isSelected()) {
            return this.getIcon(abstractButton, abstractButton.getDisabledSelectedIcon(), icon, 520);
        }
        return this.getIcon(abstractButton, abstractButton.getDisabledIcon(), icon, 8);
    }
    
    private int getTextShiftOffset(SynthContext synthContext) {
        AbstractButton abstractButton = (AbstractButton)synthContext.getComponent();
        ButtonModel model = abstractButton.getModel();
        if (model.isArmed() && model.isPressed() && abstractButton.getPressedIcon() == null) {
            return synthContext.getStyle().getInt(synthContext, this.getPropertyPrefix() + "textShiftOffset", 0);
        }
        return 0;
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        if (component.getComponentCount() > 0 && component.getLayout() != null) {
            return null;
        }
        AbstractButton abstractButton = (AbstractButton)component;
        SynthContext context = this.getContext(component);
        Dimension minimumSize = context.getStyle().getGraphicsUtils(context).getMinimumSize(context, context.getStyle().getFont(context), abstractButton.getText(), this.getSizingIcon(abstractButton), abstractButton.getHorizontalAlignment(), abstractButton.getVerticalAlignment(), abstractButton.getHorizontalTextPosition(), abstractButton.getVerticalTextPosition(), abstractButton.getIconTextGap(), abstractButton.getDisplayedMnemonicIndex());
        context.dispose();
        return minimumSize;
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        if (component.getComponentCount() > 0 && component.getLayout() != null) {
            return null;
        }
        AbstractButton abstractButton = (AbstractButton)component;
        SynthContext context = this.getContext(component);
        Dimension preferredSize = context.getStyle().getGraphicsUtils(context).getPreferredSize(context, context.getStyle().getFont(context), abstractButton.getText(), this.getSizingIcon(abstractButton), abstractButton.getHorizontalAlignment(), abstractButton.getVerticalAlignment(), abstractButton.getHorizontalTextPosition(), abstractButton.getVerticalTextPosition(), abstractButton.getIconTextGap(), abstractButton.getDisplayedMnemonicIndex());
        context.dispose();
        return preferredSize;
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        if (component.getComponentCount() > 0 && component.getLayout() != null) {
            return null;
        }
        AbstractButton abstractButton = (AbstractButton)component;
        SynthContext context = this.getContext(component);
        Dimension maximumSize = context.getStyle().getGraphicsUtils(context).getMaximumSize(context, context.getStyle().getFont(context), abstractButton.getText(), this.getSizingIcon(abstractButton), abstractButton.getHorizontalAlignment(), abstractButton.getVerticalAlignment(), abstractButton.getHorizontalTextPosition(), abstractButton.getVerticalTextPosition(), abstractButton.getIconTextGap(), abstractButton.getDisplayedMnemonicIndex());
        context.dispose();
        return maximumSize;
    }
    
    protected Icon getSizingIcon(AbstractButton abstractButton) {
        Icon icon = this.getEnabledIcon(abstractButton, abstractButton.getIcon());
        if (icon == null) {
            icon = this.getDefaultIcon(abstractButton);
            return icon;
        }
        return icon;
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((AbstractButton)propertyChangeEvent.getSource());
        }
    }
}
